;-----------------------------------------------------------;
;                                                           ;
;Maths Pack by James Vernon                                 ;
;Calculate the area of a triangle given three sides,        ;
; three vertices, SAS, or ASA.                              ;
;By: Andrey Gorlin <ag_silver_83p@ameritech.net>            ;
;                                                           ;
;-----------------------------------------------------------;

triAreaName:
    .db     "Triangle Area Finder",0

triArea:
    set     trigdeg,(iy+trigflags)
    call    choice
    ld      hl,strTriAreaChoose     ;Prompt string
    bcall(_puts)
triAreaMenu:
    call    waitkey
    cp      GK_Y
    jr      z,triAreaSides
    cp      GK_WINDOW
    jp      z,triAreaVertices
    cp      GK_ZOOM
    jp      z,triAreaSAS
    cp      GK_TRACE
    jp      z,triAreaASA
    cp      GK_MODE
    jr      nz,triAreaMenu
    ret

triAreaSides:
    call    cls
    ld      hl,strTriAreaSides      ; Display:
    call    putsNewLine             ;"Enter lengths of three sides."
    jp      heroSolve               ;Will solve there.

triAreaVertices:
    call    cls
    ld      hl,strTriAreaVertices   ; Display:
    call    pNLpromptX              ;"Enter 3 points"
    ld      de,op1                  ;"in (X,Y) form."
    call    getXpromptY             ;"Vertex 1:" / "X="    
    ld      de,taY1
    call    getNumberNL
    ld      hl,strTriAreaV2         ; Display:
    call    pNLpromptX              ;"Vertex 2:" / "X="
    ld      de,op2
    call    getXpromptY
    ld      de,taY2
    call    getNumberNL
    ld      hl,strTriAreaV3         ; Display:
    call    pNLpromptX              ;"Vertex 3:" / "X="
    ld      de,op3
    call    getXpromptY
    ld      de,taY3
    call    getNumberNL
    call    pushOps
    call    taLoadY3                ;Formula used:
    bcall(_fpmult)                  ;1/2abs((x1y2 + x2y3 + x3y1) - (x2y1 + x3y2 + x1y3))
    call    saveop1f9
    call    taLoadY2
    call    popOp1
    bcall(_fpmult)
    call    loadop2f9
    bcall(_fpsub)
    call    popOp2
    bcall(_op1exop2)
    call    pushOp1
    call    pushOp2
    call    taLoadY1
    bcall(_fpmult)
    call    saveop1f9
    call    popOp1
    call    taLoadY3
    bcall(_fpmult)
    call    loadop2f9
    bcall(_fpsub)
    call    popOp2
    bcall(_fpadd)
    call    pushOp1
    call    popOp3
    bcall(_op3toop1)
    call    taLoadY2
    bcall(_fpmult)
    call    saveop1f9
    call    popOp3
    bcall(_op3toop1)
    call    taLoadY1
    bcall(_fpmult)
    call    loadop2f9
    bcall(_fpsub)
    bcall(_op1toop2)
    call    popOp1
    bcall(_fpadd)
    bcall(_ckop1fp0)
    jr      z,triAreaVError
    call    cls
    bcall(_timespt5)                    ;Area found.
    ld      hl,strTriArea
    call    putsNewLine
    bcall(_fpsquare)                    ;Take absolute value.
    jp      dispSquaredAnsWK

triAreaVError:
    ld      hl,strTriAreaColl
    jp      putsWaitKey

triAreaSAS:
    call    cls
    ld      hl,strTriAreaSAS            ; Display:
    call    putsNewLine                 ;"Side-Angle-Side"
    ld      hl,side1
    call    putsNewLine                 ;"Side 1:"
    ld      de,op2
    call    getNumberNL
    call    ckOP2Pos
    call    pushOp2
    ld      hl,strTriAreaIncAngle       ; Display:
    call    putsNewLine                 ;"Included Angle:"
    ld      de,op1
    call    getNumberNL
    call    triAreaCkAngle
    bcall(_sin)
triAreaSASCont:
    call    saveop1f9
    ld      hl,side2                    ; Display:
    call    putsNewLine                 ;"Side 2:"
    ld      de,op1
    call    getNumberNL
    call    ckOP1Pos
    call    cls
    ld      hl,strTriArea
    call    putsNewLine
    call    popOp2
    bcall(_fpmult)                      ;Formula for SAS:
    bcall(_timespt5)                    ; 1/2ab sin C
    call    loadop2f9
    bcall(_fpmult)
    bcall(_fpsquare)
    jp      dispSquaredAnsWK

triAreaASA:
    call    cls
    ld      hl,strTriAreaASA            ; Display:
    call    putsNewLine                 ;"Angle-Side-Angle"
    ld      de,op2                      ;"Angle 1:"
    call    getNumberNL
    call    pushOp2
    bcall(_op2toop1)
    call    triAreaCkAngle
    ld      hl,strTriAreaIncSide        ; Display:
    call    putsNewLine                 ;"Included Side:"
    ld      de,op1
    call    getNumberNL
    call    ckOP1Pos
    bcall(_fpsquare)
    bcall(_timespt5)
    call    pushOp1
    ld      hl,strTriAreaAngle2         ; Display:
    call    putsNewLine                 ;"Angle 2:"
    ld      de,op3
    call    getNumberNL
    call    pushOp3                     ;Formula for ASA:
    bcall(_op3toop1)                    ; D = 180 - (A + B)
    call    triAreaCkAngle              ; c^2 sin A sin B
    call    popOp2                      ; ---------------
    bcall(_fpsub)                       ;     2 sin D
    bcall(_ckop1pos)
    jr      nz,triAreaASAError
    bcall(_ckop1fp0)
    jr      nz,triAreaASACont
triAreaASAError:
    ld      hl,strTriAreaASANotTri
    jp      putsWaitKey

triAreaASACont:
    call    cls
    ld      hl,strTriArea
    call    putsNewLine
    bcall(_sin)
    bcall(_op1toop2)
    call    popOp1
    bcall(_fpdiv)
    call    saveop1f9
    call    popOp2
    bcall(_op2toop1)
    bcall(_sin)
    call    loadop2f9
    bcall(_fpmult)
    call    saveop1f9
    call    popOp3
    bcall(_op3toop1)
    bcall(_sin)
    call    loadop2f9
    bcall(_fpmult)
    bcall(_fpsquare)
    jp      dispSquaredAnsWK


triAreaCkAngle:
    call    triAreaAngleCk
    call    saveop1f9
    call    op1set180
    call    loadop2f9
    bcall(_fpsub)
    call    triAreaAngleCk
    ret

triAreaAngleCk:
    bcall(_ckop1pos)
    jr      nz,triAreaAngleError
    bcall(_ckop1fp0)
    ret     nz
triAreaAngleError:
    pop     hl
    pop     hl
    ld      hl,strTriAreaAngleError
    bcall(_puts)
    jp      waitkey
    
taS1    =   float2
taS2    =   float3
taS3    =   float4

taLoadS1 =  f2toop2
taLoadS2 =  f3toop2
taLoadS3 =  f4toop2

taLoadY1 =  f1toop2
taLoadY2 =  f2toop2
taLoadY3 =  f3toop2

taY1    =   float1
taY2    =   float2
taY3    =   float3

strTriAreaChoose:
    .db     "F1: Sides       "
    .db     "F2: Vertices    "
    .db     "F3: Side-Angle-S"
    .db     "F4: Angle-Side-A",0

strTriAreaSides:
    .db     "Enter lengths of"
    .db     "three sides.",0

strTriAreaVertices:
    .db     "Enter 3 points  "
    .db     "in (X,Y) form.  "
    .db     "Vertex 1:",0
strTriAreaX = strXequ
;   .db     "X=",0

strTriAreaV2:
    .db     "Vertex 2:",0

strTriAreaV3:
    .db     "Vertex 3:",0

strTriAreaColl:
    .db     "The points are  "
    .db     "collinear.",0

strTriArea  = area
;   .db     "Area:",0

strTriAreaSAS:
    .db     "Side-Angle-Side",0

strTriAreaIncAngle:
    .db     "Included Angle:",0

strTriAreaAngleError:
    .db     "0 < Angle < 180.",0

strTriAreaASA:
    .db     "Angle-Side-Angle"
    .db     "Angle 1:",0

strTriAreaIncSide:
    .db     "Included Side:",0

strTriAreaAngle2:
    .db     "Angle 2:",0

strTriAreaASANotTri    = strHero
;   .db     "Not a triangle.",0

.end

